#ifndef __GUN_PORT_H__
#define __GUN_PORT_H__
#include <stdio.h>
#include <stdint.h>
#include "adcgpio.h"
#include "adc081c.h"
#include "ads1115.h"
#include "ccu_define.h"

int gunA_set_K1K2_state(void* self, int on, int timeout);
int gunA_get_PluginHomeState(void* self);
int gunA_get_FuseState(void* self);
void gunA_set_Aux12V_state(void* self, int on);
int gunA_set_ELock_state(void* self, int on, int timeout);
float gunA_get_CC1_voltage(void* self);
float gunA_get_CP_voltage(void* self, uint32_t *cp_state);
float gunA_get_PP_voltage(void* self);
float gunA_get_GunLine_temp(void* self, int idx);
void gunA_set_PWM_ratio(void* self, int ratio);
void gunA_set_LED_state(void *self, int LEDID, int on);
void gunA_set_LED_toggle(void *self, int LEDID);
int gunA_BSP_init(void);
void gunA_BSP_exit(void);

int gunB_set_K1K2_state(void* self, int on, int timeout);
int gunB_get_PluginHomeState(void* self);
int gunB_get_FuseState(void* self);
void gunB_set_Aux12V_state(void* self, int on);
int gunB_set_ELock_state(void* self, int on, int timeout);
float gunB_get_CC1_voltage(void* self);
float gunB_get_CP_voltage(void* self, uint32_t *cp_state);
float gunB_get_PP_voltage(void* self);
float gunB_get_GunLine_temp(void *self, int idx);
void gunB_set_PWM_ratio(void* self, int ratio);
void gunB_set_LED_state(void *self, int LEDID, int on);
void gunB_set_LED_toggle(void *self, int LEDID);
int gunB_BSP_init(void);
void gunB_BSP_exit(void);


#endif//__GUN_PORT_H__
